/*
* Project name:
      Can_Spi_2nd (CAN Network demonstration with mikroE's CAN-SPI module)
* Copyright:
      (c) Mikroelektronika, 2015.
* Description:
      This project is a simple demonstration of using CAN-SPI module; with minor
      adjustments, it should work with any other MCU that has a SPI module.
      This code demonstrates how to use CANSPI library functions and procedures.
      It is used together with the Can_Spi_1st example (on second MCU), and it can
      be used to test the connection of MCU to the CAN network.
      This node initiates the communication with the Can_2nd node by sending some
      data to its address. The 2nd node responds by sending back the data incre-
      mented by 1. This (1st) node then does the same and sends incremented data
      back to 2nd node, etc.
* Test configuration:
        MCU:             FT900
                         http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
        Dev.Board:       Clicker 2 for FT900
                         http://www.mikroe.com/ft90x/clicker-2/
        Oscillator:      CPU at full system clock, 100.000MHz
        Ext. modules:    ac:CANSPI   - canspi click 3.3v
                         ac:BarGraph - bargraph click
        SW:              mikroC PRO for FT90x
                         http://www.mikroe.com/mikroc/FT90x/
* NOTES:
      - Consult the CAN standard about CAN bus termination resistance.
      - CANSPI click is connected to mikroBUS 1.
      - BarGraph click board is connected to mikroBUS 2.
*/

unsigned char Can_Init_Flags, Can_Send_Flags, Can_Rcv_Flags; // can flags
unsigned char Rx_Data_Len;                                   // received data length in bytes
char RxTx_Data[8];                                           // can rx/tx data buffer
char Msg_Rcvd;                                               // reception flag
const long ID_1st = 12111, ID_2nd = 3;                       // node IDs
long Rx_ID;

// CANSPI module connections
sbit CanSpi_CS            at  GPIO_PIN28_bit;
sbit CanSpi_Rst           at  GPIO_PIN47_bit;
// End CANSPI module connections
sbit BarGraph_CS at GPIO_PIN33_bit;      // Chip Select pin for BarGraph Click

// hc chip select function
void hc_cs_fnc(void) {
  BarGraph_CS = 0;
  asm{nop};
  asm{nop};
  asm{nop};
  BarGraph_CS = 1;
}

// display graph on Bar Graph click
void BarGraph_DisplayPort(int value) {
  char temp1, temp2;
  unsigned int temp;

  temp1 = (0x00FF & value);
  temp2 = (0x0300 & value) >> 8;

  SPIM_Wr_Ptr(temp2);
  SPIM_Wr_Ptr(temp1);
  hc_cs_fnc();
}

void main() {
  GPIO_Digital_Output(&GPIO_PORT_32_39, _GPIO_PINMASK_1); // set BarGraph Chip Select as digital output

  Can_Init_Flags = 0;                                         //
  Can_Send_Flags = 0;                                         // clear flags
  Can_Rcv_Flags  = 0;                                         //

  Can_Send_Flags = _CANSPI_TX_PRIORITY_0 &                    // form value to be used
                   _CANSPI_TX_XTD_FRAME &                     // with CANSPIWrite
                   _CANSPI_TX_NO_RTR_FRAME;

  Can_Init_Flags = _CANSPI_CONFIG_SAMPLE_THRICE &             // form value to be used
                   _CANSPI_CONFIG_PHSEG2_PRG_ON &             // with CANSPIInit
                   _CANSPI_CONFIG_XTD_MSG &
                   _CANSPI_CONFIG_DBL_BUFFER_ON &
                   _CANSPI_CONFIG_VALID_XTD_MSG &
                   _CANSPI_CONFIG_LINE_FILTER_OFF;

  // Initialize SPI1 module
  SPIM1_Init_Advanced(_SPI_MASTER_CLK_RATIO_128, _SPI_CFG_POLARITY_IDLE_LOW | _SPI_CFG_PHASE_CAPTURE_FALLING, _SPI_SS_LINE_NONE);

  CANSPIInitialize(1,3,3,3,1,Can_Init_Flags);                           // initialize external CANSPI module
  CANSPISetOperationMode(_CANSPI_MODE_CONFIG,0xFF);                     // set CONFIGURATION mode
  CANSPISetMask(_CANSPI_MASK_B1,-1,_CANSPI_CONFIG_XTD_MSG);             // set all mask1 bits to ones
  CANSPISetMask(_CANSPI_MASK_B2,-1,_CANSPI_CONFIG_XTD_MSG);             // set all mask2 bits to ones
  CANSPISetFilter(_CANSPI_FILTER_B2_F3,ID_1st,_CANSPI_CONFIG_XTD_MSG);  // set id of filter B2_F3 to 1st node ID

  CANSPISetOperationMode(_CANSPI_MODE_NORMAL,0xFF);                           // set NORMAL mode

  while (1) {                                                                 // endless loop
    Msg_Rcvd = CANSPIRead(&Rx_ID , RxTx_Data , &Rx_Data_Len, &Can_Rcv_Flags); // receive message
    if ((Rx_ID == ID_1st) && Msg_Rcvd) {                                      // if message received check id
      BarGraph_DisplayPort(RxTx_Data[0]);                                     // id correct, output data to BarGraph
      RxTx_Data[0]++ ;                                                        // increment received data
      CANSPIWrite(ID_2nd, RxTx_Data, 1, Can_Send_Flags);                      // send incremented data back
    }
  }
}