/*
 * Project name:
     FLASH Demo
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
     This is a simple demonstration how to use internal flash memory to
     store data. The data is being written starting from the given location;
     then, the same locations are read and the data is displayed via UART terminal.
 * Test configuration:
     MCU:             FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       Clicker 2 for FT900
                      http://www.mikroe.com/ft90x/clicker-2/
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    UART click - ac:UART_Click
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Initial delay of 5 seconds is put at the beginning, for purposes of giving the debugger more than enough time to access semaphore
 */

#include <stdint.h>

#define ARRSIZE 256

typedef enum LineEnding
{
  WIN, // windows ending
  UNIX // UNIX ending
} LineEnding;

LineEnding
  linEnd = WIN;

uint8_t
  buff[ARRSIZE],
  txt [ARRSIZE];

uint32_t
  i,
  flashStatus,
  flashAddr = 0x10000;

/**
 * Sends the line along with its terminating characters
 * @param text     pointer to string to be sent
 * @param linEnd   choose the line ending either for WIN or UNIX
 */
void UART1_Write_Line(uint8_t* text,
                    LineEnding linEnd)
{
  if (linEnd == WIN)
  {
    UART1_Write_Text(text);
    UART1_Write(13);
    UART1_Write(10);
  }
  else
  {
    UART1_Write_Text(text);
    UART1_Write(10);
  }
}

void main()
{
  Delay_ms(5000);
  UART1_Init(56000); // Initialize UART module at 56000 bps
  Delay_ms  (100);   // Wait for UART module to stabilize

  UART1_Write_Line("FLASH test", linEnd);
  UART1_Write_Line("Start",      linEnd);

  if (Flash_wait_semaphore() == 0)
  {
    UART1_Write_Text("Reading values from the address: ");
    LongWordToStr(flashAddr,txt);
    UART1_Write_Line(txt, linEnd);

    flashStatus = Flash_Read_Buffer(flashAddr, buff, sizeof(buff));
    if (flashStatus == 0)
    {
      for (i = 0; i < sizeof(buff)-1; ++i)
      {
        ByteToStr(buff[i],txt);
        UART1_Write_Text(txt);
        UART1_Write_Text(", ");
      }
      ByteToStr(buff[sizeof(buff)-1],txt);
      UART1_Write_Text(txt);
      UART1_Write_Line(".", linEnd);
    }
    else
    {
      LongWordToStr(flashStatus,txt);
      UART1_Write_Text("Reading failed! Error: ");
      UART1_Write_Line(txt, linEnd);
    }


    UART1_Write_Line("", linEnd);
    UART1_Write_Line("Filling the buffer...", linEnd);
    // Writing values into array
    for (i = 0; i < sizeof(buff); ++i)
    {
      buff[i] = i;
    }

    UART1_Write_Line("", linEnd);
    UART1_Write_Text("Writing values from the buffer to the address: ");
    LongWordToStr(flashAddr,txt);
    UART1_Write_Line(txt, linEnd);

    flashStatus = Flash_Write_Buffer(flashAddr, buff, sizeof(buff));
    if (flashStatus == 0)
    {
      // Clear the array first
      for (i = 0; i < sizeof(buff); i++)
      {
              buff[i] = 0;
      }

      UART1_Write_Line("", linEnd);
      UART1_Write_Line("Buffer erased.", linEnd);
      UART1_Write_Line("", linEnd);
      UART1_Write_Text("Reading back values into buffer from the address: ");
      LongWordToStr(flashAddr,txt);
      UART1_Write_Text(txt);
      UART1_Write_Line(":", linEnd);

      flashStatus = Flash_Read_Buffer(flashAddr, buff, sizeof(buff));

      if (flashStatus == 0)
      {
        for (i = 0; i < sizeof(buff)-1; ++i)
        {
          ByteToStr(buff[i],txt);
          UART1_Write_Text(txt);
          UART1_Write_Text(", ");
        }
        ByteToStr(buff[sizeof(buff)-1],txt);
        UART1_Write_Text(txt);
        UART1_Write_Line(".", linEnd);
      }
      else
      {
        LongWordToStr(flashStatus,txt);
        UART1_Write_Text("Reading failed! Error: ");
        UART1_Write_Line(txt, linEnd);
      }
    }
    else
    {
      LongWordToStr(flashStatus,txt);
      UART1_Write_Text("Writing failed! Error: ");
      UART1_Write_Line(txt, linEnd);
    }

    flashStatus = Flash_Erase_Sector(flashAddr);

    if (flashStatus == 0)
    {
      UART1_Write_Line("", linEnd);
      UART1_Write_Line("Erase success!", linEnd);
      UART1_Write_Line("", linEnd);
      UART1_Write_Text("Again reading values from the address: ");
      LongWordToStr(flashAddr,txt);
      UART1_Write_Text(txt);
      UART1_Write_Line(":", linEnd);

      flashStatus = Flash_Read_Buffer(flashAddr, buff, sizeof(buff));
      if (flashStatus == 0)
      {
        for (i = 0; i < sizeof(buff)-1; ++i)
        {
          ByteToStr(buff[i],txt);
          UART1_Write_Text(txt);
          UART1_Write_Text(", ");
        }
        ByteToStr(buff[sizeof(buff)-1],txt);
        UART1_Write_Text(txt);
        UART1_Write_Line(".", linEnd);
      }
      else
      {
        LongWordToStr(flashStatus,txt);
        UART1_Write_Text("Reading failure! Error: ");
        UART1_Write_Line(txt, linEnd);
      }
    }
    else
    {
      LongWordToStr(flashStatus,txt);
      UART1_Write_Text("Erase failure! Error: ");
      UART1_Write_Line(txt, linEnd);
    }
    // The test is done
    Flash_release_semaphore();
  }
  else
  {
    UART1_Write_Line("Cannot acquire semaphore", linEnd);
  }
  UART1_Write_Line("", linEnd);
  UART1_Write_Line("", linEnd);
  UART1_Write_Line("Test completed.", linEnd);
}