/*
 * Project name:
       RS485_Master (RS485 Library demo - Master side)
   Copyright:
       (c) Mikroelektronika, 2015.
 * Description:
      This is a simple demonstration on how to use the mikroC's RS485 library.
      It is being used in pair with the RS485_Slave project. Master (this
      machine) initiates communication with slave by sending 1 byte of data to
      the slave with designated slave address (160). The slave accepts data,
      increments it and sends it back to the master.
      The data received and Error are shown on BarGraph.
      Several situations are shown here:
        - RS485 Master Init sequence;
        - Data sending master-to-slave with designated slave address;
        - Data sending master-to-slave with broadcast slave address (50);
        - Handling of unsuccessful master-slave communication (connection reset);
      Also shown here, but not strictly related to RS485 library, is:
        - Function calling from the interrupt routine - which data is to be saved,
          and how.
      For further explanations on RS485 library, please consult the mikroC Help.

 * Test configuration:
         MCU:             FT900
                          http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
         Dev.Board:       Clicker 2 for FT900
                          http://www.mikroe.com/ft90x/clicker-2/
         Oscillator:      CPU at full system clock, 100.000MHz
         Ext. modules:    ac:RS485 - rs485 click 3.3v
                          ac:BarGraph - bargraph click
         SW:              mikroC PRO for FT90x
                          http://www.mikroe.com/mikroc/FT90x/

 * NOTES:
       - Initialize the UART module before performing RS485 init.
       - RS485 click board is connected to mikroBUS 1.
 */

char dat[10];               // buffer for receving/sending messages
char i,j;

sbit RS485_rxtx_pin  at GPIO_PIN57_bit;  // set transcieve pin
sbit BarGraph_CS at GPIO_PIN33_bit;      // Chip Select pin for BarGraph Click

// Interrupt routine
void interrupt() iv IVT_UART1_IRQ {
  RS485Master_Receive(dat);
}

// hc chip select function
void hc_cs_fnc(void) {
  BarGraph_CS = 0;
  asm{nop};
  asm{nop};
  asm{nop};
  BarGraph_CS = 1;
}

// display graph on Bar Graph click
void BarGraph_DisplayPort(int value) {
  char temp1, temp2;
  unsigned int temp;

  temp1 = (0x00FF & value);
  temp2 = (0x0300 & value) >> 8;

  SPIM_Wr_Ptr(temp2);
  SPIM_Wr_Ptr(temp1);
  hc_cs_fnc();
}

void main(){
  long cnt = 0;
  char val = 0;
  char errVal = 0;
  int maskVal = 0;

  GPIO_Digital_Output(&GPIO_PORT_32_39, _GPIO_PINMASK_1); // set BarGraph Chip Select as digital output
  // initialize SPI
  SPIM1_Init();
  // initialize UART
  UART1_Init(19200);
  Delay_ms(100);

  RS485Master_Init();                // initialize MCU as Master

  IRQ_CTRL.B31 = 0;
  UART1_LCR.B7 = 0;
  UART1_INT_ENABLE = 0x1;  //Enable Received Data Available Interrupt

  dat[0] = 0xAA;
  dat[1] = 0xF0;
  dat[2] = 0x0F;
  dat[4] = 0;                      // ensure that message received flag is 0
  dat[5] = 0;                      // ensure that error flag is 0
  dat[6] = 0;

  RS485Master_Send(dat, 1, 160);

while (1) {                            // upon completed valid message receiving
                                       //   data[4] is set to 255
    cnt++;
    if (dat[5]) {                      // if an error detected, signal it
      BarGraph_DisplayPort(0xAA);       // by setting value of AA to BarGraph
    }
    if (dat[4]) {                      // if message received successfully
      cnt = 0;
      dat[4] = 0;                      // clear message received flag
      j = dat[3];
      for (i = 1; i <= dat[3]; i++) {  // show data on BarGraph
        BarGraph_DisplayPort(dat[i-1]);
      }                                // increment received dat[0]
      dat[0] = dat[0]+1;               // send back to master
      Delay_ms(1);
      RS485Master_Send(dat,1,160);
    }
    if (cnt > 100000) {
      errVal++;
      maskVal = ~maskVal;
      BarGraph_DisplayPort(maskVal);
      delay_ms(250);
      cnt = 0;
      RS485Master_Send(dat,1,160);
      if (errVal > 10)                 // if sending failed 10 times
        RS485Master_Send(dat,1,50);    // send message on broadcast address
        errVal = 0;
    }
  }
}