/*
** can.h
**
** Copyright © 2013 Future Technology Devices International Limited
**
** Company Confidential
**
** Header file for FT900 CAN driver
**
** Author: Huynh Le Trung (based on Teng Peow's verification)
** Project: FT900
** Module: FT900 CAN driver 
** Requires:
** Comments:
**
** History:
**  1 - initial version
*/

#ifndef __FT900_CAN_H_
#define __FT900_CAN_H_

#include "stdint.h"
#include "__Can_Hw_Map.h"

// ============================================================================================== //
//                                     R E G I S T E R S                                                                      //
// ============================================================================================== //
#define MR                                0x00                // mode register
#define CMD                                0x01                // command register
#define SR                                0x02                // status register
#define ISR_IACK                0x03                // interrupt status / acknowledge register
#define IMR                                0x04                // interrupt mask register
#define RMC                                0x05                // receive message counter register
#define BTR0                        0x06                // bus timing register 0
#define BTR1                        0x07                // bus timing register 1
#define TXBUF0                        0x08                // transmit buffer register 0
#define TXBUF1                        0x09                // transmit buffer register 1
#define TXBUF2                        0x0A                // transmit buffer register 2
#define TXBUF3                        0x0B                // transmit buffer register 3
#define RXBUF0                        0x0C                // receive buffer register 0
#define RXBUF1                        0x0D                // receive buffer register 1
#define RXBUF2                        0x0E                // receive buffer register 2
#define RXBUF3                        0x0F                // receive buffer register 3
#define ACR0                        0x10                // acceptance code register 0
#define ACR1                        0x11                // acceptance code register 1
#define ACR2                        0x12                // acceptance code register 2
#define ACR3                        0x13                // acceptance code register 3
#define AMR0                        0x14                // acceptance mask register 0
#define AMR1                        0x15                // acceptance mask register 1
#define AMR2                        0x16                // acceptance mask register 2
#define AMR3                        0x17                // acceptance mask register 3
#define ECC                                0x18                // error code capture register
#define RXERR                        0x19                // RX error counter register
#define TXERR                        0x1A                // TX error counter register
#define ALC                                0x1B                // management address register         

// ============================================================================================== //
//                                     R E G I S T E R    B I T S                                                      //
// ============================================================================================== //
// MR bits
#define MR_RM                        0x04
#define MR_LOM                        0x02
#define MR_AFM                        0x01
#define MR_NORMAL                0x00

// CMR bits
#define CMD_TR                        0x04
#define CMD_AT                        0x02

// SR bits
#define SR_RBS                        0x80
#define SR_DSO                        0x40
#define SR_TBS                        0x20
#define SR_RS                        0x08        
#define SR_TS                        0x04
#define SR_ES                        0x02
#define SR_BS                        0x01

// ISR_IACK bits
#define ISR_IACK_ALI        0x40
#define ISR_IACK_EWI        0x20
#define ISR_IACK_EPI        0x10
#define ISR_IACK_RI                0x08
#define ISR_IACK_TI                0x04
#define ISR_IACK_BEI        0x02
#define ISR_IACK_DOI        0x01

// IMR bits
#define IMR_ALIM                0x40
#define IMR_EWIM                0x20
#define IMR_EPIM                0x10
#define IMR_RIM                        0x08
#define IMR_TIM                        0x04
#define IMR_BEIM                0x02
#define IMR_DOIM                0x01

// ECC bits
#define ECC_RXWRN                0x80
#define ECC_TXWRN                0x40
#define ECC_EDIR                0x20
#define ECC_ACKER                0x10
#define ECC_FRMER                0x08
#define ECC_CRCER                0x04
#define ECC_STFER                0x02
#define ECC_BER                        0x01

// SYS_REGMSC0CFG_B3 bits
#define CAN1_SLOW                0x01
#define CAN0_SLOW                0x02

// ============================================================================================== //
//                                E X P O R T E D   C O N S T A N T S                                                     //
// ============================================================================================== //
// Status messages
typedef enum
{
        CAN_OK = 0,
        CAN_ERROR,
        CAN_RX_OVERRUN
} CAN_STATUS;

// CAN mode definitions
typedef enum 
{
        CAN_MODE_NORMAL,
        CAN_MODE_RESET, 
        CAN_MODE_LISTEN_ONLY
} TCANModeEnum;

// CAN frame format definitions
typedef enum 
{
        CAN_STD_FRAME,
        CAN_EXT_FRAME
} TCANFrameFormatEnum;


// CAN interrupt definitions
#define CAN_ARB_LOST_INT                0x40
#define CAN_ERR_WARNING_INT                0x20
#define CAN_ERR_PASSIVE_INT                0x10
#define CAN_RX_INT                                0x08
#define CAN_TX_INT                                0x04
#define CAN_BUS_ERR_INT                        0x02
#define CAN_DATA_OVERRUN_INT        0x01

// Status bits for function canGetStatus()
#define INIT 0x01
#define CERR 0x02
#define ERPA 0x04
#define RXOR 0x08
#define TXOR 0x10
#define BOFF 0x80

// Option bits for setting filter
#define DEFAULT                                        0x00
#define DISABLE_FILTER                        0x01
#define DUAL_FILTER                                0x02
#define FILTER_2                                0x04
#define EXTENDED_FRAME                        0x08
#define INCLUDE_RTR                                0x10
#define RTR_1                                        0x20

// Data structure for a single CAN message 
typedef struct
{
        TCANFrameFormatEnum frameFormat;        // CAN frame format (standard / extended)
        uint8_t               RTR;                // RTR
        uint32_t              ID;                 // Message identifier
        uint8_t               len;                // Data length (0-8)
        uint8_t               buf[8];             // Data buffer
} TCANMsg;

// Basic configuration structure declaration  uint16_t bitrate;        // bitrate in kb/s, allowed values: 1000, 800, 500, 250, 125, 50, 25, 10


// ============================================================================================== //
//                              F U N C T I O N   P R O T O T Y P E S                                                     //
// ============================================================================================== //
/*--------------------------------------------------------------------------------------------------
 Function used for initializing system registers.
 
 Parateres:
 CANx - used for selecting CAN device (CAN1 / CAN2)
--------------------------------------------------------------------------------------------------*/
void CANSysInit(TCANStruct *CANx);

/*--------------------------------------------------------------------------------------------------
 Function initializes CAN device. This is basic initialization which sets mode of operation.

 Parameters:
 CANx - used for selecting CAN device (CAN1 / CAN2)
 conf - pointer TCANBasicConfig structure which contains settings
 
 Returns:
 CAN_OK    - CAN device was initialized
 CAN_ERROR - error occured, CAN device was not initialized
--------------------------------------------------------------------------------------------------*/
CAN_STATUS canInit(TCANStruct *CANx, uint16_t bitratekB);

/*--------------------------------------------------------------------------------------------------
 Function used for reading (any) CAN registers.

 Parameters:
 CANx - used for selecting CAN device (CAN1 / CAN2)
 reg  - CAN register

 Returns register value.
--------------------------------------------------------------------------------------------------*/
uint8_t readCANReg(TCANStruct *CANx, uint8_t reg);

/*--------------------------------------------------------------------------------------------------
 Function used for writing (any) CAN registers.

 Parameters:
 CANx - used for selecting CAN device (CAN1 / CAN2)
 val  - register value
 reg  - CAN register
--------------------------------------------------------------------------------------------------*/
void writeCANReg(TCANStruct *CANx, uint8_t val, uint8_t reg);

/*--------------------------------------------------------------------------------------------------
 Function enables CAN interrupt.
 
 Parameters:
 CANx - used for selecting CAN device (CAN1 / CAN2)
 ints - the interrupts to be enabled
--------------------------------------------------------------------------------------------------*/
void canEnableInt(TCANStruct *CANx, uint8_t ints);

/*--------------------------------------------------------------------------------------------------
 Function disables CAN interrupt.

 Parameters:
 CANx - used for selecting CAN device (CAN1 / CAN2)
 ints - the interrupts to be disabled
--------------------------------------------------------------------------------------------------*/
void canDisableInt(TCANStruct *CANx, uint8_t ints);

/*--------------------------------------------------------------------------------------------------
 Function clears CAN interrupt status flag.
 
 Parameters:
 CANx - used for selecting CAN device (CAN1 / CAN2)
 ints - the interrupts to be disabled
--------------------------------------------------------------------------------------------------*/
void canClearIntFlag(TCANStruct *CANx, uint8_t ints);

/*--------------------------------------------------------------------------------------------------
 Function returns the global status variable.
 
 Parameters:
 CANx - used for selecting CAN device (CAN1 / CAN2)
 
 Returns an 8-bit status. The bits in the status are:
        0: INIT   - 1 after a completed initialization; 0 if not yet initialized or init failed
        1: CERR   - 1 if a CAN bit or frame error occured
        2: ERPA   - 1 if a CAN "error passive" occured
        3: RXOR   - 1 if a receive queue overrun occured
        4: TXBUSY - 1 if the transmit buffer is being locked for MCU
        5: Reserved
        6: Reserved
        7: BOFF   - 1 if a CAN "bus off" error occured
--------------------------------------------------------------------------------------------------*/
uint8_t canGetStatus(TCANStruct *CANx);

/*--------------------------------------------------------------------------------------------------
 Function sets the CAN bus speed.
 
 Parameters:
 CANx    - used for selecting CAN device (CAN1 / CAN2)
 bitrate - the desired bitrate
 
 Notes: Bitrate is in kb/s, allowed values: 1000, 500, 250, 125, 50, 25, 10
--------------------------------------------------------------------------------------------------*/
void canSetSpeed(TCANStruct *CANx, uint16_t bitrate);

/*--------------------------------------------------------------------------------------------------
 Funtion sets CAN filter.
 
 Parameters:
 CANx    - used for selecting CAN device (CAN1 / CAN2)
 canID   - ID to be received by filter
 options - options for setting the filter. The options are:
        DEFAULT: Filter is enabled, standard frame, single filter config, RTR bit is not included
        DISABLE_FILTER: Filter is disabled
        DUAL_FILTER: If this option is selected, use dual filter config. Otherwise use single filter.
        FILTER_2: Effective only when DUAL_FILTER is selected. If chosen, set filter 2, else filter 1.
        EXTENDED_FRAME: If this option is selected, set the filter for extended frame.
        INCLUDE_RTR: If this option is selected, RTR bit will be included in the filter.
        RTR_1: Effective only when INCLUDE_RTR is selected. If chosen, RTR = 1, else RTR = 0.
 
 Returns: 
 CAN_OK    - filter was set 
 CAN_ERROR - could not set filter
--------------------------------------------------------------------------------------------------*/
CAN_STATUS canSetFilter(TCANStruct *CANx, uint32_t canID, uint8_t options);

/*--------------------------------------------------------------------------------------------------
 Function implements a CAN transmit queue. With each function call aa message is added to the queue.
 
 Parameters:
 CANx         - used for selecting CAN device (CAN1 / CAN2)
 pTransmitBuf - data structure with message to be send

 Returns: 
 CAN_OK    - message was added to the transmit queue
 CAN_ERROR - queue is full, message was not added, bit TXOR in canGetStatus() set
 
 Notes: For many applications a queue with length "1" will be sufficient. Only applications
        with a high busload or very slow bus speed might need a queue of length "3" or more.
--------------------------------------------------------------------------------------------------*/
CAN_STATUS canPushMessage(TCANStruct *CANx, TCANMsg *pTransmitBuf);

/*--------------------------------------------------------------------------------------------------
 Function implements a CAN receive queue. With each function call a message is pulled from the queue.
 
 Parameters:
 CANx        - used for selecting CAN device (CAN1 / CAN2)
 pReceiveBuf - data structure with message received
 
 Returns:
 CAN_OK         - message was pulled from receive queue
 CAN_ERROR      - queue empty, no message received
 CAN_RX_OVERRUN - RX queue overrun occured
--------------------------------------------------------------------------------------------------*/
CAN_STATUS canPullMessage(TCANStruct *CANx, TCANMsg *pReceiveBuf);

/*--------------------------------------------------------------------------------------------------
 Function gets the number of messages currently in the RX queue.
 
 Parameters:
 CANx        - used for selecting CAN device (CAN1 / CAN2)
 
 Returns:
 The number of messages in the RX queue.
--------------------------------------------------------------------------------------------------*/
uint8_t canRXQueueCount(TCANStruct *CANx);

#endif // __FT900_CAN_H_